@@ -1,4 +1,4 @@
-/* $Id: main.c,v 1.45 2004/12/14 18:47:12 vnoel Exp $ */
+/* $Id: main.c,v 1.46 2005/01/24 17:54:32 vnoel Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -16,6 +16,7 @@
 #endif
 
 #include <glib/goption.h>
+#include <glib/gmem.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-client.h>
 #include "dict.h"
@@ -23,6 +24,8 @@
 #include "gdict-applet.h"
 #include "gdict-app.h"
 
+extern GSList *gdict_windows;
+
 static gboolean
 save_yourself_cb (GnomeClient       *client,
                   gint               phase,
@@ -32,15 +35,22 @@ save_yourself_cb (GnomeClient       *client,
                   gint               fast,
                   gpointer           client_data)
 {
-	gchar *argv[] = {NULL, NULL, NULL, NULL, NULL};
+	gchar **argv;
 	gchar *word;
-	gint argc = 1;
-	GDictWindow *gdict = client_data;
+  GSList *tmp = gdict_windows;
+	GDictWindow *gdict;
+  int i, argc;
+
+  argc = g_slist_length(gdict_windows + 1);
+  argv = g_malloc (sizeof(gchar *) * argc);
 
 	argv[0] = g_get_prgname ();
-
-	if ((word = gdict_defbox_get_word(gdict->defbox)) != NULL)
-		argv[argc++] = word;
+  for (i=1; i < argc; i++) {
+    gdict = tmp->data;
+    word = gdict_defbox_get_word (gdict->defbox);
+    argv[i] = word;
+    tmp = g_slist_next (tmp);
+  }
 
 	gnome_client_set_restart_command(client, argc, argv);
 	gnome_client_set_clone_command(client, 0, NULL);
@@ -91,20 +101,22 @@ int main (int argc, char *argv[])
 	}
 
 	ctx = g_option_context_new (" - Dictionary and spelling tool");
+  g_option_context_set_ignore_unknown_options (ctx, TRUE);
 	g_option_context_parse (ctx, &argc, &argv, &error);
 
 	if (argc > 1) {
-		for (i = 1; i < argc; i++) {			
-			window = gdict_window_new ();
-			gdict = GDICT_WINDOW (window);
-			gtk_window_present (GTK_WINDOW(gdict));
-			gdict_defbox_lookup(gdict->defbox, (char *)argv[i]);
+		for (i = 1; i < argc; i++) {
+      if (argv[i] != NULL && g_ascii_isalpha (argv[i][0])) {
+        window = gdict_window_new ();
+        gdict = GDICT_WINDOW (window);
+        gtk_window_present (GTK_WINDOW(gdict));
+        gdict_defbox_lookup(gdict->defbox, (char *)argv[i]);
+      }
 		}
 	} else {
 		window = gdict_window_new ();
 		gdict = GDICT_WINDOW (window);
 		gtk_window_present (GTK_WINDOW(gdict));
-		
 	}
 
 	gtk_main();
